﻿// // ******************************************************************
// //       /\ /|       @File         protocol_Usb_enums.h
// //       \ V/        @Brief
// //       | "")       @Author        lijinwen, ghz005@uni-trend.com.cn
// //       /  |        @Creation      2024-05-17
// //      /  \\        @Modified      2024-06-24
// //    *(__\_\
// // ******************************************************************

#pragma once
#include <cstdint>

namespace Protocol
{
    class UsbEnums
    {
    public:
        enum class EventInfoTitles
        {
            UNKNOW,
            INDEX,
            START_TIME,
            SYNC,
            SOF,
            SETUP,
            PID,
            DATA0,
            DATA1,
            DATA2,
            MDATA,
            ADDR,
            ACK,
            //IN,关键字
            TIN,
            //OUT,关键字
            TOUT,
            NAK,
            STALL,
            NYET,
            FNUM,
            CRC5,
            CRC16,
            EOP,
            //ERROR, 关键字
            ERR,
        };

        enum class UsbSpeed
        {
            LOW_SPEED = 0,
            FULL_SPEED = 1,
            HIGH_SPEED = 2,
        };

        enum class UsbFieldType
        {
            NO_DEFINE = 0,
            SOP,
            SYNC,
            PID,
            ADDR,
            FRAME,
            DATA,
            CRC
        };

        enum class UsbPacketType
        {
            NO_DEFINE = 0,
            TOKEN,
            DATA,
            HAND_SHAKE,
            SPECIAL,
        };

        enum class Condition
        {
            SYNC,
            RESET,
            PAUSE,
            RESUME,
            PACKAGE_END,
            TOKEN_PACKAGE,
            HANDSHAKE_PACKAGE,
            DATA_PACKAGE,

            SPECIAL,
            ERR,//ERROR是常量 ljw
        };

        enum class SignalRate
        {
            LOW_RATE,
            FULL_RATE,
            HIGH_RATE,
        };

        enum class TokenPackageType //: uint8_t
        {
            OUT_TYPE = 0b0001,//1 // OUT 是常量 ljw
            SOF = 0b0101,  //5
            IN_TYPE = 0b1001,   //9 // IN 是常量 ljw
            SETUP = 0b1101,//13
        };

        enum class SpecialPacketType //: uint8_t
        {
            // Pre = 0b1100,
            ERR = 0b1100,
            SPLIT = 0b1000,
            PING = 0b0100,
            REMAIN = 0b000,
        };

        enum class HandshakePackageType //: uint8_t
        {
            ACK = 0b0010,   //2
            NAK = 0b1010,   //10
            STALL = 0b1110, //14
            NYET = 0b0110,  //6
        };

        enum class DataPackageType //: uint8_t
        {
            DATA0 = 0b0011, //3
            DATA1 = 0b1011, //11
            DATA2 = 0b0111, //7
            M_DATA = 0b1111, //15
        };

        enum class ErrorPackageType
        {
            PIDCRC,
            CRC5,
            CRC16,
            BIT_FILL,
        };

        enum class DataRelation
        {
            LT,
            GT,
            LTEQ,
            GTEQ,
            EQ,
            NEQ,
            IN_TYPE,//  IN是常量 ljw
            OUT_TYPE,//  OUT是常量 ljw
        };
    };
}
